#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_03/b_15/sys/src/install/perl/SAPDB/Install/App/DebugPackage.pm#1 $
# $DateTime: 2008/01/18 16:05:39 $
# $Change: 173096 $
#
# Desc: install debug packages afterwards 
#




package SAPDB::Install::App::DebugPackage;

use SDB::Common::Getopt::Long;
use SAPDB::Install::System;
use SAPDB::Install::Registry;
use SAPDB::Install::PackageCollector;
use SAPDB::Install::InstallRegistry;
use SAPDB::Install::Log;
use SAPDB::Install::StdIO;
use SAPDB::Install::Tools;
use SAPDB::Install::MD5Sum;
use SAPDB::Install::Cleaner;



sub print_usage{
	require SAPDB::Install::Version;
	print "\nMaxDB Installer \nVersion: ".$SAPDB::Install::Version::Buildstring."\n";
	print $SAPDB::Install::Version::Copyright."\n\n";
	
	print "usage: SDBINST -debugpackage [all|<package 1>[,package 2,...]] [-l|--list]\n".
		  "               [-L] [-h|--help] [-a <directory>]\n\n\n";
	print "  -debugpackage     select debugpackage to install\n\n";
	print "  -l                list all found debug packages\n\n";
	print "  -L                list installed packages, which will be upgraded\n\n";
	print "  -a <directory>    search for debug packages in <directory>\n\n";
	return 1;
}

sub main{
	local @ARGV = @_;
	my $rc = 0;
	my ($opt_debugpackages,
		$opt_archive_dir,
		$opt_help,
		$opt_list,
		$opt_L);
		
	my %opt_ctrl = (
		'debugpackage=s' => sub {$opt_debugpackage = [split (',',$_[1])]},
		'archive_dir=s' => \$opt_archive_dir,
		'a=s' => \$opt_archive_dir,
		'l' => \$opt_list,
		'list' => \$opt_list,
		'L' => \$opt_L,
		'h' => \$opt_help,
		'help' => \$opt_help
	);	
	
	SDB::Common::Getopt::Long::Configure ('no_pass_through','no_ignore_case');	
	
	if (!GetOptions(%opt_ctrl) || $opt_help){
		print_usage();
		return 0;
	}	
	
	if (defined $opt_archive_dir && ! -d $opt_archive_dir){
		print2stderr ("wrong usage: -opt_archive_dir: directory doesn\'t exist\n");
		print_usage  ();
		return -1;
	}
	
	if (!defined $opt_debugpackage and !defined $opt_list){
		print2stderr ("wrong usage: please add an argument to option -debugpackage\n");
		print_usage  ();
		return -1;	
	}
	
	my $log = SAPDB::Install::Log->new();
	$log->LogOff (1);
	
	my $collector = SAPDB::Install::PackageCollector->new ();
	
	$collector->log ($log);
	
	$collector->archive_dir (defined $opt_archive_dir ? $opt_archive_dir : $SAPDB::Install::Config{'CallerDir'});
	
	unless (defined $collector->GenDebugPackageIndex()){
		print2stderr ("Cannot find debug packages in ".$collector->archive_dir ()."\n");
		return -1;
	}
		
	if ($opt_list){
		print2stdout ("Found MaxDB debug package(s) for:\n");
		foreach my $package (sort keys (%{$collector->debugpackage_index ()})){
			foreach my $checksum (keys %{$collector->debugpackage_index ()->{$package}}){
				print2stdout ($package." ".$collector->debugpackage_index ()->{$package}->{$checksum}->{'BUILD_STRING'}.
					  ' (Make id = '.$collector->debugpackage_index ()->{$package}->{$checksum}->{'MAKE_ID'}.")\n"); 
			}
		}
		return 0;			
	}	
	
	unless(IsAdmin()){
		print STDERR "please restart as ".($^O =~ /mswin/i ? 'local administrator' : 'root')."!\n";
		return -1;
	}
	
	my ($indep_data) = readIndepPath ();
	
	unless (-d $indep_data){
		print2stderr ("Cannot find IndepData path\n");
		return -1;
	}
	
	unless (-f "$indep_data/config/install/INSTREG"){
		print2stderr ("Ccannot find InstallRegistry\n");
		return -1;
	}
	
	my $instreg = SAPDB::Install::InstallRegistry->new ("$indep_data/config/install",1,$log);
	
	unless (defined $instreg){
		print2stderr ("Cannot get InstallRegistry\n");
		return -1;
	}
	
	unless (defined $collector->GenRegPackageIndex ($instreg)){
		print2stderr ("cannot get installed packages\n");
		return -1;
	}
	
	my @packages = ();
	
	foreach my $package (keys (%{$instreg->RegData->{'HashRef_Packages'}})){
		unless(exists $collector->debugpackage_index()->{$package}){
			next;
		}
		if ($opt_debugpackage->[0] ne 'all'){
			my $found = 0;
			foreach my $dbgpack (@$opt_debugpackage){
				if ($dbgpack eq $package){
					$found = 1;
					last;
				}
			}
			next unless $found;
		}
		
		foreach my $instkey (keys (%{$instreg->RegData->{'HashRef_Packages'}->{$package}})){
			
			if (exists $instreg->RegData->{'HashRef_Packages'}->{$package}->{$instkey}->{'DbgFiles'} and 
				defined $instreg->RegData->{'HashRef_Packages'}->{$package}->{$instkey}->{'DbgFiles'}){
				
				my $dbglist_ok = 1;
				
				foreach my $dbgfile (@{$instreg->RegData->{'HashRef_Packages'}->{$package}->{$instkey}->{'DbgFiles'}}){
					unless (exists $instreg->RegData->{'HashRef_Packages'}->{$package}->{$instkey}->{'FileList'}->{$dbgfile} and
							-f "$instkey/$dbgfile"){
						$log->SetMsg ("MSG: Package \"$package\" [$instkey] already has debug files, but file $dbgfile is missing\n");
						$dbglist_ok = 0;
						last;
					}					
				}
				
				if($dbglist_ok){
					$log->SetMsg ("MSG: Package \"$package\" [$instkey] already has debug files, skipping it\n");
					next;
				}
			}
			
			my $md5sum = $instreg->RegData->{'HashRef_Packages'}->{$package}->{$instkey}->{'CheckSum'};
			
			if (exists $collector->debugpackage_index()->{$package}->{$md5sum}){
				push @packages,{'name'=> $package, 'instkey' => $instkey,'data' => 
								$collector->debugpackage_index()->{$package}->{$md5sum}};
			}	
		}
	}

	if ($#packages == -1){
		print2stdout ("no debug package to install\n");
		return 0;
	}


	if ($opt_L){
		print2stdout ("Found MaxDB debug package(s) match for:\n");
		foreach my $package (@packages){
			print $package->{'name'}." [".$package->{'instkey'}."] ".$package->{'data'}->{'BUILD_STRING'}."\n"; 
		}
		return 0;			
	}			
	
	print2stdout ("installing debug package(s) for: \n");
	
	local $cleaner = SAPDB::Install::Cleaner->new();
				
	foreach my $package (@packages){
		my $packobj = $instreg->getPackage ($package->{'name'},$package->{'instkey'});
		
		unless (defined $packobj){
			print2stderr ("cannot get package [$package->{name},$package->{instkey}]\n");
			return -1;
		}
		print2stdout ($package->{'name'}." [".$package->{'instkey'}."] ".$package->{'data'}->{'BUILD_STRING'}."\n");		
		my $untgz = $package->{'data'}->{'UNTGZ'};
		my $filelist = $package->{'data'}->{'FILELIST_VALUE'};
		$untgz->SetDestDir ($package->{'instkey'});
		if ($^O !~ /mswin/i){
			my $uid = getpwnam ($packobj->User);
			unless (defined $uid and $uid >= 0){
				print2stderr ("User ".$packobj->User." doesn\'t exist");
				return -1;	
			}
			my $gid = getgrnam ($packobj->Group);
			unless (defined $gid and $gid >= 0){
				print2stderr ("Group ".$packobj->Group." doesn\'t exist");
				return -1;	
			}			
			$untgz->SetOwner ($uid,$gid);			
		}
		$instreg->DenyWrite(0);
		$packobj->setValid(0);
		
		my %left_files = %$filelist;
	
		while (1){
			my $header = $untgz->Next ();
			unless (defined $header){
				if ($untgz->GetErr){
					print2stderr ("Error during extraction of ".$package->{'data'}->{'ARCHIVE_NAME'}.': '.join(': ',$untgz->GetErr)."\n");
					return -1;
				}
				# got eof 
				# rewind archive if there is another package to extend 
				$untgz->Rewind();
				$untgz->Next();	# skip PACKAGADATA
				$untgz->Next();	# skip filelist			
				last;
			}
			my ($file) = ($header =~ /([^\ ]*)$/);
			
			$file =~ s/^\.\///; 
		
			if (exists $filelist->{$file}){
				$log->SetMsg ("MSG: $header\n");
				$untgz->ExtractFile ();
				my $checksum = MD5Sum ($package->{'instkey'}.'/'.$file);
				unless ($checksum eq $filelist->{$file}){
					print2stderr ("mutation of file $file\n");
					return -1;
				}
				delete $left_files{$file};
				$packobj->FileList()->{$file} = $filelist->{$file};
			}
		}
		my @missed_files = keys (%left_files);
	
		unless ($#missed_files == -1){
			print2stderr ("there are file(s) missing in debugpackage:\n");	
			foreach my $file (@missed_files){
				print2stderr ("$file\n");
			}
			return -1;	
		}
	
		$packobj->DbgFiles ([keys (%$filelist)]);
		if (exists $package->{'data'}->{'SIZE'} and defined $package->{'data'}->{'SIZE'}){
				$packobj->Size ($packobj->Size + $package->{'data'}->{'SIZE'});
		}
		$packobj->setValid(1);
		print2stdout ("Done\n");			
	}
	 
	return $rc;
}

1;
