#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_03/b_15/sys/src/install/perl/SAPDB/Install/Instance/Check/Instances.pm#1 $
# $DateTime: 2008/01/18 16:05:39 $
# $Change: 173096 $
#

package SAPDB::Install::Instance::Check::Instances;

use SAPDB::Install::Instance::Base;
use SAPDB::Install::Instance::Check::Conditions;

push @ISA, 
	'SAPDB::Install::Instance::Base',
	'SAPDB::Install::Instance::Check::Conditions';

sub instances {
	my ($self) = @_;
	my $dbm = $self->{dbm};

	#
	# find out instance type if not already known
	#
	unless (defined $self->{'instancetype'}) {
		my $rc = $self->conditions ();
		return undef unless (defined $rc && $rc == 0);
	}

	$self->set_errorstate ('ERROR');
	$self->msgbegin ("looking for running instances");	
	$self->msg0 ("looking for running instances...\n");
	$self->{'running_instances'} = undef;

	my $enum = $dbm->db_enum ();
	unless (defined $enum) {
		$self->msg1 ("error during db_enum\n");
		$self->msg1 ("\n");
		$self->msg1 ($dbm->lastdialog ());
		return undef;
	}

	my $lines = $#{$enum->{'DBNAME'}} + 1;
	if ($lines <= 0) {
		$self->msg1 ("no instances found\n");
		$self->msg1 ("\n");
		$self->msg1 ($dbm->lastdialog ());
		return undef;
	}

	my @running_instances = ();
	for (my $i = 0; $i < $lines; $i++) {
		next unless ($enum->{'STATE'}->[$i] eq 'running');
		next if ($enum->{'DBNAME'}->[$i] eq $self->{'dbname'});
		
		push @running_instances, $enum->{'DBNAME'}->[$i];
	}

	if (($#running_instances + 1) != 0) {
		$self->{running_instances} = \@running_instances;

		$self->msg1 ("found running instances\n");
		foreach (@{$self->{running_instances}}) {
			$self->msg1 ("  ".$_."\n");
		}
	}

	$self->set_errorstate ('OK');
	$self->msgend ();
	return 0;
} 


sub instance_version{
  my ($this,$dbenum,$dbname,$version) = @_;
  return $this->instance_attribute
    ($dbenum,$dbname,$version,'VERSION'); 
}

sub instance_path{
  my ($this,$dbenum,$dbname,$path) = @_;
  return $this->instance_attribute
    ($dbenum,$dbname,$path,'PATH'); 
}

sub instance_state{
  my ($this,$dbenum,$dbname,$state) = @_;
  return $this->instance_attribute
    ($dbenum,$dbname,$state,'STATE'); 
}
sub instance_attribute{
  my ($this,$dbenum,$dbname,$attval,$attribute) = @_;
  my $retVal=0;
  my $index=-1;
  my $dbs = $dbenum->{'DBNAME'};
  my $counter=0;
  foreach my $db(@$dbs){
    if($db eq $dbname){
      $index=$counter;
      last;
    }
	$counter++;
  }
  if($index == -1){
    return "Database $dbname not found";
  }
  $$attval=$dbenum->{$attribute}->[$index];
#  print "$attribute=$$attval\n";
  return $retVal;
}







1;


# Local Variables: ***
# compile-command:  "C:/Programme/j2sdk1.4.2_04/bin/java -Dvalidate-xml=true -classpath  \"c:/SapDevelop/DEV/develop/test/jtest/tests/launcher.jar\" com.sap.sapdb.launcher.AntCmdLauncher2 \"c:/Documents and Settings/d022195/My Documents/RunnerStarter/Repository/perl-test.xml\" script" ***
# End: ***
