import os
import sys
import string
import re
import socket
from sdb import  dbm

###########################################################################################

# initialisation:

ARGV2 = sys.argv[1:]
PROG     = ARGV2[0]
FIRSTARG =''
ALLARGS = ARGV2[1:]
if len(ARGV2) >= 2:
   FIRSTARG = ARGV2[1]
RESTARGS  = ARGV2[2:]
if sys.platform[:3] == 'win':
   JAVABASENAME = 'java'
   JAVAWBASENAME = 'javaw'
   SHELLSUPPRESSOUTPUT = ' > nul 2>&1'
   PATHSEP = ';'
   SPAWNQUOTE = '"'
   # a perl 'runtime' (run a main method by providing the --main <classname> option)
   # which does not open its own console under windows:
   SDBPERL = 'sdbregview.exe'
   import _winreg
   try: 
      SYNCMANCONFIGDIR  = os.path.join(_winreg.QueryValueEx(_winreg.OpenKey(_winreg.HKEY_CURRENT_USER ,"Volatile Environment"),"APPDATA")[0], 'sdb')
      SYNCMANCONFIG  = os.path.join(SYNCMANCONFIGDIR, 'syncman.cfg')
   except EnvironmentError:
      print "Error reading Registry"
      sys.exit(1)
else:
   JAVABASENAME = 'java'
   JAVAWBASENAME = 'java'
   SHELLSUPPRESSOUTPUT = ' > /dev/null 2>&1'
   PATHSEP = ':'
   SPAWNQUOTE = ''
   # a perl 'runtime' (run a main method by providing the --main <classname> option):
   SDBPERL = 'SDBINST'
   try:
      SYNCMANCONFIGDIR = os.path.join(os.getenv('HOME'), '.sdb')
      SYNCMANCONFIG  = os.path.join(SYNCMANCONFIGDIR, 'syncman.cfg')
   except:
      pass
_dbmsource = dbm.DBM ('', '', '','')
INDEPPROG  = re.split('\n', _dbmsource.cmd('dbm_getpath     INDEPPROGPATH'))[0]
INDEPDATA  = re.split('\n', _dbmsource.cmd('dbm_getpath     INDEPDATAPATH'))[0]
JDBCINST   = os.path.join(INDEPPROG, 'runtime', 'jar')
SYNCINST   = os.path.join(INDEPPROG, 'app', 'syncman')
JAVA       = ''
JAVAW      = ''
J2EEPATH   = ''
XSERVER    = os.path.join(INDEPPROG, 'bin', 'x_server')
# the user may place custom jar files and shared objects (e.g. rcp jars and .so/.dll or JDBC drivers) here,
# these will override files with same names in the original installation:
EXTLIB = os.path.join(INDEPDATA, 'app', 'syncman', 'extern')
CUSTOMCLASSPATH = ''
if os.path.exists(EXTLIB):
   for _entry in os.listdir(EXTLIB):
      if _entry != 'j2ee.jar':
         CUSTOMCLASSPATH = CUSTOMCLASSPATH + os.path.join(EXTLIB, string.strip(_entry)) + PATHSEP

############################################################################################

def set_java_env():
   global JAVA
   global JAVAW
   global J2EEPATH
   cFileObj = file(SYNCMANCONFIG, 'r')
   for line in cFileObj.readlines():
      name, value = string.split(line, '=')
      name = string.strip(name)
      value = string.strip(value)
      if name == 'JAVAPATH':
         JAVAW = os.path.join(value, JAVAWBASENAME)
         JAVA = os.path.join(value, JAVABASENAME)
      elif name == 'J2EEPATH':
         J2EEPATH = value
   cFileObj.close()
   return

############################################################################################

def test_java():
   versionTestCmdl = '"' + JAVA + '"'  + ' -version'
   found = os.system(SPAWNQUOTE + versionTestCmdl + ' ' + SHELLSUPPRESSOUTPUT + SPAWNQUOTE)
   if found != 0:
      print 'java call error.'
      sys.exit(1)
   found = 1
   versFileObjList = os.popen3(versionTestCmdl)
   for line in versFileObjList[2].readlines():
      tokens = string.split(line, ' ')
      if tokens[0].lower() == 'java' and tokens[1].lower() == 'version':
         found = 0
         break
   for fileObj in versFileObjList:
      fileObj.close()
   if found != 0:
      print 'java call error.'
      sys.exit(1)
   versionList = string.split(string.strip(tokens[2], '"'+string.whitespace), '.')
   if string.atoi(versionList[0]) != 1 or string.atoi(versionList[1]) < 4:
      print 'wrong java version.\nyou have ' + line + 'you need java 1.4 or greater.'
      sys.exit(1)
   return

############################################################################################

# config('JAVA')
# config('MSGSERVER')
# config('SYNCSERVICE', '-d', dbname)

def config(what, *rest):
   if what == 'JAVA':
      if not os.path.exists(SYNCMANCONFIGDIR):
         os.makedirs(SYNCMANCONFIGDIR)
      configFileArg = '--configfile ' + '"' + SYNCMANCONFIG + '"'
   else:
      configFileArg = ''
   configurator =  '"' + os.path.join(INDEPPROG, 'bin', SDBPERL) + '"' + ' --main SAPDB::Install::User::SyncMan::Config::main'
   cmdl = configurator + ' ' + configFileArg + ' -config ' + what + ' ' + string.join(rest)
   # under windows python seems to do funky things to the commandline
   # (i.e. trims quotesigns at start and end) before it passes
   # it to libc's 'system()' function. We have to provide
   # an additional layer of quotesigns:
   retcode = os.system(SPAWNQUOTE + cmdl + SPAWNQUOTE)
   if retcode != 0:
      print 'cannot create configuration.'
      sys.exit(1)
      
############################################################################################
      
# operate_syncservice(FIRSTARG, '-A', accesskey, string.join(RESTARGS))
# operate_syncservice('start', '-A', accesskey, string.join(RESTARGS))

def operate_syncservice(what, *rest):
   startSyncservice = ('"' + JAVA + '"' +
   ' -Xmx256m' +
   ' -classpath ' + '"' +
   CUSTOMCLASSPATH +
   os.path.join(SYNCINST, 'lib') + PATHSEP +
   os.path.join(JDBCINST, 'sapdbc.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'syncservice.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'msgserverclient.jar') + PATHSEP +
   os.path.join(J2EEPATH, 'j2ee.jar') + '"' +
   ' com.sap.sdb.syncMan.SyncService')
   
   stopSyncservice = ('"' + JAVA + '"' +
   ' -classpath ' +  '"' +
   CUSTOMCLASSPATH +
   os.path.join(SYNCINST, 'lib') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'syncservice.jar') + '"' +
   ' com.sap.sdb.syncMan.SyncServiceAdmin' +
   ' shutdown')

   args = string.join(rest)

   if what == 'start':
      cmdl = startSyncservice + ' ' + args
      retcode = os.system(SPAWNQUOTE + cmdl + SPAWNQUOTE)
   elif what == 'stop':
      cmdl = stopSyncservice + ' ' + args + ' ' + SHELLSUPPRESSOUTPUT
      retcode = 0
      retcode = os.system(SPAWNQUOTE + cmdl + SPAWNQUOTE)
      if retcode != 0:
         print 'could not stop syncservice or syncservice not running.'
         sys.exit(1)
      else:
         print 'syncservice stopped.'
         sys.exit(0)
   elif what == 'list':
      retcode = syncmanacc('-A', rest[1], '-c', 'list')
      sys.exit(retcode)
   else:
      print 'usage: syncservice (start|stop|list|config) -d <database instance>'
      sys.exit(1)
      
############################################################################################

# retcode = syncmangui(RESTARGS)

def syncmangui(*args):
   # spawn* peculiarity under windows:
   # the 'path' parameter _must_not_ be quoted, even if it contains
   # blanks in its path.
   # in the argument list, every element _must_ be qouted, even if it does not contain
   # blanks.
   fixedArgs = [JAVAW,
   '-Djava.library.path=' + EXTLIB + PATHSEP + os.path.join(SYNCINST, 'lib'),
   '-classpath' ,
   (CUSTOMCLASSPATH +
   os.path.join(SYNCINST, 'lib') + PATHSEP +
   os.path.join(JDBCINST, 'sapdbc.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'syncservice.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'msgserverclient.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'runtime.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'osgi.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'jface.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'jfacetext.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'swt.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'text.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'syncmangui.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'swt-pi.jar') + PATHSEP +
   os.path.join(J2EEPATH, 'j2ee.jar')),
   'com.sap.sdb.syncMan.gui.SyncManGUI']
   spawnargs = quoteArgList(fixedArgs) + list(args)
   #print spawnargs
   os.spawnv(os.P_NOWAIT, JAVAW, spawnargs)
      
############################################################################################

# retcode = syncmanacc('-A', 'MESSAGESERVER', '-c', 'exists')
# retcode = syncmanacc('-A', accesskey, '-c', 'exists')
# retcode = syncmanacc(RESTARGS)
# retcode = syncmanacc('-A', rest[1], '-c', 'list')
# retcode = syncmanacc('-A', 'MESSAGESERVER', '-c', 'list')

def syncmanacc(*args):
   startSyncmanacc = ('"' + JAVA + '"' +
   ' -classpath ' +  '"' +
   CUSTOMCLASSPATH +
   os.path.join(SYNCINST, 'lib') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'syncmanacc.jar') + '"' +
   ' com.sap.sdb.syncMan.acc.SyncManACC')
   cmdl = startSyncmanacc + ' ' + string.join(args)
   if not 'list' in args:
      cmdl = cmdl + ' ' + SHELLSUPPRESSOUTPUT
   retcode = os.system(SPAWNQUOTE + cmdl + SPAWNQUOTE)
   return retcode
   
############################################################################################

# operate_msgserver(FIRSTARG, string.join(RESTARGS))
# operate_msgserver('start', string.join(RESTARGS))

def operate_msgserver(what, *rest):
   startMsgserver = ('"' + JAVA + '"' +
   ' -Xmx256m' +
   ' -classpath ' +  '"' +
   CUSTOMCLASSPATH +
   os.path.join(SYNCINST, 'lib') + PATHSEP +
   os.path.join(JDBCINST, 'sapdbc.jar') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'msgserver.jar') + PATHSEP +
   os.path.join(J2EEPATH, 'j2ee.jar') + '"' +
   ' com.sap.sdb.msgServer.Server' +
   ' -A  MESSAGESERVER' +
   ' -f ' + '"' + os.path.join(SYNCINST, 'MSGSERVER.xml') + '"')
   
   stopMsgserver = ('"' + JAVA + '"' +
   ' -classpath ' +  '"' +
   CUSTOMCLASSPATH +
   os.path.join(SYNCINST, 'lib') + PATHSEP +
   os.path.join(SYNCINST, 'lib', 'msgserverclient.jar') + '"' +
   ' com.sap.sdb.msgServer.AdminClient' +
   ' -A MESSAGESERVER shutdown')

   args = string.join(rest)

   if what == 'start':
      cmdl = startMsgserver + ' ' + args
      retcode = os.system(SPAWNQUOTE + cmdl + SPAWNQUOTE)
   elif what == 'stop':
      cmdl = stopMsgserver + ' ' + SHELLSUPPRESSOUTPUT
      retcode = 0
      retcode = os.system(SPAWNQUOTE + cmdl + SPAWNQUOTE)
      if retcode != 0:
         print 'could not stop msgserver or msgserver not running.'
         sys.exit(1)
      else:
         print 'msgserver stopped.'
         sys.exit(0)
   elif what == 'list':
      retcode = syncmanacc('-A', 'MESSAGESERVER', '-c', 'list')
      sys.exit(retcode)
   else:
      print 'usage: msgserver (start|stop|list|config)'
      sys.exit(1)
      
############################################################################################

# retcode = createexampledatabases()

def createexampledatabases(*args):
   dbmUser = 'DBM'
   maxUsertasks = '70'
   dbaUser = 'DBADMIN'
   retcode = 0
   for dbname in ['DBMASTER', 'DBCLIENT']:
      if args[0] == '"-drop"':
         removeDB(dbname, dbmUser)
      elif string.strip(args[0]) == '':
         createDB(dbname, dbmUser, maxUsertasks, dbaUser)
      else:
         print 'Unknown option: ' + args[0]
         retcode = 1
   return retcode
   
############################################################################################

# retcode = createexampletables()

def createexampletables(*args):
   startCreateTables = ('"' + JAVA + '"' +
   ' -classpath ' +  '"' +
   os.path.join(JDBCINST, 'sapdbc.jar') + PATHSEP +
   os.path.join(SYNCINST, 'example', 'lib', 'syncmanexample.jar') + '"' +
   ' com.sap.sdb.syncMan.test.CreateExampleTables')
   cmdl = (startCreateTables + ' ' +
   '-masterURL "jdbc:sapdb://127.0.0.1/DBMASTER" ' +
   '-masterUser DBSERVICE ' +
   '-masterPassword SECRET ' +
   '-clientURL "jdbc:sapdb://127.0.0.1/DBCLIENT" ' +
   '-clientUser DBSERVICE ' +
   '-clientPassword SECRET ' +
   string.join(args))
   retcode = os.system(SPAWNQUOTE + cmdl + SPAWNQUOTE)
   return retcode
   
############################################################################################

def dbmcmd (session, cmd):
    try:
        session.cmd(cmd)
    except dbm.DBMServError, err:
        print 'ERR at DBMServer command ' + cmd + ': ' + str (err)
        sys.exit(1)

############################################################################################

def createDB(dbname, dbmUser, maxUsertasks, dbaUser):
    node = ''
    session = dbm.DBM(node, '')
    #
    # create database
    #
    print 'Create database ' + dbname
    dbmcmd(session, 'db_create ' + dbname + ' ' + dbmUser + ',' + dbmUser)
    #
    # init kernel parameters
    #
    print '    Create param file'
    dbmcmd(session, 'param_startsession')
    dbmcmd(session, 'param_init')
    dbmcmd(session, 'param_put MAXUSERTASKS ' + maxUsertasks)
    #
    dbmcmd(session, "param_checkall");
    dbmcmd(session, "param_commitsession");
    print 'OK'
    #
    # set devspaces
    #
    print '    Add devspaces'
    dbmcmd(session, 'param_addvolume 1 LOG  LOG_001  F  2000 ');
    dbmcmd(session, 'param_addvolume 1 DATA DAT_001  F 10000 ');
    print 'OK'
    #
    # start database
    #
    print '    Start database ' + dbname
    dbmcmd(session, 'db_cold')
    dbmcmd(session, 'db_activate ' + dbaUser + ',' + dbaUser)
    dbmcmd(session, 'load_systab -ud domp')
    dbmcmd(session, 'util_execute SET LOG AUTO OVERWRITE ON')

############################################################################################

def removeDB(dbname, dbmUser):
    node = ''
    session = dbm.DBM(node, dbname, '', dbmUser+','+dbmUser)
    print 'Stop database ' + dbname
    dbmcmd(session, 'db_offline')
    print '    Drop database ' + dbname
    dbmcmd(session, 'db_drop')

############################################################################################

# converts:
# [] -> ''
# [''] -> '""'
# ['', ''] -> '"" ""'
# ['a', 'xyz', 'p'] -> '"a" "xyz" "p"'

def quoteArgs(argList):
   if len(argList) == 0:
      return ''
   return '"' + string.join(argList, '" "') +'"'

############################################################################################

# converts:
# [] -> []
# [''] -> []
# ['', ''] -> []
# ['a', '', 'p'] -> ['"a"', '"p"']

def quoteArgList(argList):
   retval = []
   for entry in argList:
      if not len(entry) == 0:
         retval.append(SPAWNQUOTE + entry + SPAWNQUOTE)
   return retval

############################################################################################


def startXServer():
   cmdl = '"' + XSERVER + '"'  + ' start'
   os.system(SPAWNQUOTE + cmdl + ' ' + SHELLSUPPRESSOUTPUT + SPAWNQUOTE)

############################################################################################




# main part:

if PROG == 'syncmanjconf':
   config('JAVA')
   set_java_env()
   test_java()
   sys.exit(0)
if PROG == 'createexampledatabases':
   startXServer()
   retcode = createexampledatabases(quoteArgs(ALLARGS))
   sys.exit(retcode)
if not os.path.exists(SYNCMANCONFIG):
   config('JAVA')
set_java_env()
test_java()
if PROG == 'msgserver':
   if FIRSTARG != 'config':
      retcode = syncmanacc('-A', 'MESSAGESERVER', '-c', 'exists')
      if retcode != 0:
         print """Generating access key 'MESSAGESERVER',
since it does not exist yet.
Please specify the following parameters:"""
         config('MSGSERVER')
      startXServer()
      if 'start' == FIRSTARG or 'stop' == FIRSTARG or 'list' == FIRSTARG:
         operate_msgserver(FIRSTARG, quoteArgs(RESTARGS))
      else:
         operate_msgserver('start', quoteArgs(RESTARGS))
   else:
      config('MSGSERVER')
elif PROG == 'syncservice':
   if not '-d' in RESTARGS or RESTARGS.index('-d') == len(RESTARGS) - 1:
      print 'usage: syncservice (start|stop|list|config) -d <database instance>'
      sys.exit(1)
   dindex = RESTARGS.index('-d')
   dbname = RESTARGS[dindex + 1]
   del RESTARGS[dindex]
   del RESTARGS[dindex]
   hostname = string.split(socket.gethostname(), '.')[0]
   accesskey = string.upper('SYNCSERVICE' + '_' + hostname + '_' + dbname)
   if FIRSTARG != 'config':
      retcode = syncmanacc('-A', accesskey, '-c', 'exists')
      if retcode != 0:
         print 'Generating access key ' + accesskey + """,
since it does not exist yet.
Please specify the following parameters:"""
         config('SYNCSERVICE', '-d', dbname)
      startXServer()
      if 'start' == FIRSTARG or 'stop' == FIRSTARG or 'list' == FIRSTARG:
         operate_syncservice(FIRSTARG, '-A', accesskey, quoteArgs(RESTARGS))
      else:
         operate_syncservice('start', '-A', accesskey, quoteArgs(RESTARGS))
   else:
      config('SYNCSERVICE', '-d', dbname)
elif PROG == 'syncmangui':
   startXServer()
   retcode = syncmangui(quoteArgs(ALLARGS))
   sys.exit(retcode)
#elif PROG == 'syncmanacc':
#   retcode = syncmanacc(quoteArgs(ALLARGS))
#   sys.exit(retcode)
elif PROG == 'createexampletables':
   startXServer()
   retcode = createexampletables(quoteArgs(ALLARGS))
   sys.exit(retcode)
else:
   print 'unknown program name: ' + PROG
   sys.exit(1)
sys.exit(0)


